#ifdef nonce


#include <iostream>
#include <cstdlib>

#include <netdb.h>
#include <linux/socket.h>

#include "tcpconn.h"

/**
	@author fabsoft <fabsoft@gmail.com>
*/
namespace n2nc {

namespace net {
using namespace std;

/**
  Store a generic address-independent IP based address, it supports IPv4 and IPv6 versions both.
  It provides also tonet() function to retrieve network ordered format address.
  Reference:
   http://www.kame.net/newsletter/19980604/
   RFC2553
   NOTE the private varible which holds the sockaddr_xx is a sockaddr_in6 type which mantain
   interoperability with sockaddr_storage, but much smaller.
*/
class Address {
#define ADDRESS_IP_CHARSIZE (16 +7 +1)
    /**
      prii
    */
  private:
    //FIXME redundant info here, i need to keep IP in text form or socket form.
    std::string m_ipaddress ;
    /** Intentionally NOT Compliant to RFC2553, sockaddr_storage mean memory waste and
     this standard is NOT followed enough, some system(mine too) doesn't have ss_len...
     */
    struct sockaddr_in6 m_ssaddr ;
    inline int retrieve_sockaddr_by_ip(std::string &, struct sockaddr_in6 * );
    inline int retrieve_ip_by_sockaddr(const struct sockaddr_in6 *, std::string &);
  public:
    Address();

    /** Constructor takes directly a address of family AF_INET[6] or 
        fully qualified domain name in a string format
        @param address in format "127.0.0.1", "::1", "::FFFF:127.0.0.1" IPv4 in IPv6 mapped
    */
    Address( std::string address );

    /** Constructor takes directly a address of family AF_INET[6] by POSIX storage structure.
        @param addr_storage sockaddr_XXX family socket address storage.
        addr_storage is copied to mantain thread-safety.
        NOTE that the struct ss_len field defines its size, so only needed bytes are copied.
        FIXME use ss_family instead !
     */
    Address( const struct sockaddr_storage *addr_storage );

    /** Essentially free sockaddr_in6 */
    ~Address();

    /** Set the address when not using constructor, use this to avoid throw exception.
        Returns 0 if succedes, otherwise the address is invalid */
    int setAddress( std::string addr );

    /** Set the address when not using constructor, use this to avoid throw exception.
        Return false if the address is invalid .*/
    int setAddress( const struct sockaddr_storage *addr_storage );

    /** Returns the address family type eg: AF_INET */
    u_char type() ;

    /** Prints to a output stream the address fields */
    void dump_sockaddr(std::ostream &os) ;

    /** Returns a pointer to sockaddr to use with other db functions or socket(), the structure is
        internally allocated by malloc and freed by destructor.Subsequent call will return the same pointer
        be careful to NOT modify it.  //FIXME return a copy instead ?
     */
    struct sockaddr_storage *tonet();

    /** Returns a copy of the actual ip address */
    std::string tostring();

};


} //ns

} //ns

#endif
