#ifndef N2NCCONFIGURATION_H
#define N2NCCONFIGURATION_H

#include "nixsys.h"

namespace n2nc {

/**
  \brief Class to provides configuration option pair.
  @author fabsoft <fabsoft@gmail.com>
*/
class ConfigurationOpt{
public:
    /**
     * 
     * @param first The option name.
     * @param second The option value.
     */
    ConfigurationOpt(std::string first, std::string second);
    /** Copy constructor */
    ConfigurationOpt(const ConfigurationOpt& opt);
    ~ConfigurationOpt();
    std::string toString();
    std::string value();
    /** sets the associated value for this Config */
    void setValue(std::string value);
    /** sets the associated value for this Config.
        eg: ConfigurationOpt opt("key","value");
        opt = "newvalue" ;
    */
    void operator=(std::string value);
    
private:
    std::string m_first ;
    std::string m_second ;
};

/**
  \brief Class to provides configuration context.
  A context is ConfigurationOpt container.
  @author fabsoft <fabsoft@gmail.com>
*/
class ConfigurationCtx{
public:
    ConfigurationCtx(std::string name);
    ~ConfigurationCtx();
    /** To string */
    std::string toString();
    /** Adds an Option in this context */
    bool addOption(const ConfigurationOpt &opt);
    /** Deletes an Option in this context */
    bool delOption(ConfigurationOpt &opt);
    /** @return the option value associated to optname */
    ConfigurationOpt& getOpt(std::string optname);
    /** @return the option value associated to optname */
    ConfigurationOpt& operator[](std::string optname);
    

private:
    std::list<ConfigurationOpt> m_options ;
    std::string m_name ;
};

/**
  \brief Class to provides configuration handler and storage.
  This class is a ConfigurationCtx container.
  @author fabsoft <fabsoft@gmail.com>
*/
class Configuration{
public:
    Configuration();
    ~Configuration();
    /** To string. \return A string containing all contexts and their options */
    std::string toString();
    /** Loads configuration from string */
    bool fromString(std::string &str);
    /** Adds a Context */
    bool addContext(const ConfigurationCtx &ctx);
    /** Deletes a Context */
    bool delContext(ConfigurationCtx &ctx);
    /** @return The Context by its name */
    ConfigurationCtx& getCtx(std::string ctxname);
    /** @return The Context by its name. */
    ConfigurationCtx& operator[](std::string ctxname);

private:
    std::list<ConfigurationCtx> m_contexts ;
    typedef std::pair<std::string,ConfigurationCtx> m_context2_t ;
    std::map<std::string,ConfigurationCtx> m_contexts2 ;

    /**
      \example configuration_ex.cpp
      This is an example of using Configuration utility.
      more about this example
    */
};

} //ns
#endif
