#include "address_old.h"
#ifdef nonce

namespace n2nc {
namespace net {

Address::Address() {
  memset(&m_ssaddr,0,sizeof(m_ssaddr)) ;
}

Address::Address( std::string ipaddr ) {
int ret ;
  memset(&m_ssaddr,0,sizeof(m_ssaddr)) ;
  if ( ret = setAddress(ipaddr) )
    throw gai_strerror(ret);
}

Address::Address( const struct sockaddr_storage *addr_storage ){
int ret ;
  memset(&m_ssaddr,0,sizeof(m_ssaddr)) ;
  if ( ret = setAddress(addr_storage) )
    throw gai_strerror(ret);
}

Address::~Address(){
}

int Address::setAddress(std::string addr){
  int ret ;
  if ( ret = retrieve_sockaddr_by_ip(addr,&m_ssaddr) ){
    m_ipaddress.clear() ;
    return ret ;
  }
  m_ipaddress = addr ;
  return ret ;
}

int Address::setAddress( const struct sockaddr_storage *addr_storage ){
  int ret ;
  if ( ret = retrieve_ip_by_sockaddr((const struct sockaddr_in6 *)addr_storage, m_ipaddress ) ){
    m_ipaddress.clear() ;
    return ret ;
  }
  memset(&m_ssaddr,0,sizeof(m_ssaddr));
/*#if HAVE_SOCKADDR_STORAGE_LEN*/  //FIXME please sockaddr_storage sucks
  size_t siz = (addr_storage->ss_family == AF_INET ? sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6) );
  memcpy(&m_ssaddr,addr_storage,siz );
  return 0 ;
}

inline int Address::retrieve_ip_by_sockaddr(const struct sockaddr_in6 *ss,std::string &ip){
  char s_ip[ADDRESS_IP_CHARSIZE];
  int ret ;
  size_t siz = (ss->sin6_family == AF_INET ? sizeof(struct sockaddr_in) : sizeof(struct sockaddr_in6) );
  ret = getnameinfo( (struct sockaddr*)ss, siz , s_ip, ADDRESS_IP_CHARSIZE, NULL, 0, NI_NUMERICHOST );
  ip = s_ip ;
  return ret ;
}

inline int Address::retrieve_sockaddr_by_ip(std::string &ip, struct sockaddr_in6 *ss ){
  struct addrinfo hints, *ai ;
  int ret;

  memset( &hints, 0, sizeof( hints ) );
  hints.ai_flags = AI_NUMERICHOST ; //this force to not resolve hostname
  hints.ai_family = AF_UNSPEC;      //FIXME AF_UNSPEC or PF_INET ??
  hints.ai_socktype = SOCK_STREAM;

  ret = getaddrinfo( ip.c_str() , NULL , &hints, &ai );
  if ( ret ){
//    freeaddrinfo(ai);
    return ret ;
  }
  memcpy(ss,ai->ai_addr,ai->ai_addrlen);              //Shoud be ok
  freeaddrinfo(ai);
  return ret ;
}

//sa_family_t ??
u_char Address::type(){
  if (m_ipaddress.empty()){  return NULL ; /*throw ;*/ }
  return m_ssaddr.sin6_family ;
}

struct sockaddr_storage *Address::tonet() {
  if (m_ipaddress.empty()){  return NULL ; /*throw ;*/ }
  return (struct sockaddr_storage *)&m_ssaddr ;
}

std::string Address::tostring(){
  return m_ipaddress ;
}

void Address::dump_sockaddr(std::ostream &os){
char s_ip[ADDRESS_IP_CHARSIZE];
  if (m_ipaddress.empty()) { 
    os << "uninitialized\n" ;
    return ;
  }
  getnameinfo( (struct sockaddr*)&m_ssaddr, sizeof(m_ssaddr), s_ip, ADDRESS_IP_CHARSIZE, NULL, 0, NI_NUMERICHOST );
  os << "address family: " << m_ssaddr.sin6_family << (m_ssaddr.sin6_family == AF_INET ? " --> IPv4" : " --> IPv6") << endl ;
  os << "address name: " << m_ipaddress << endl ;
  os << "address name by sockaddr: " << s_ip << endl ;
  os << "**********************************************" << endl ;
}

}
}
#endif
