#include "resolver.h"

namespace n2nc {
namespace net {


Address& Resolver::getAddressByName(std::string &name, bool resolve){

}


Address* Resolver::getAddressByFQDN(std::string &addr,bool resolve)
{
  Address *t_addr;
  struct addrinfo ai,*ai_list;
  int ret;

  memset(&ai,0,sizeof(ai));
  ai.ai_flags = AI_ADDRCONFIG ;
  ai.ai_flags |= (resolve ? 0 : AI_NUMERICHOST ) ;
  ai.ai_socktype = SOCK_STREAM;

  ret = getaddrinfo(addr.c_str(),NULL,&ai,&ai_list); 
  if (ret){
    ;/* throw */
    N2NC_PRINT("getaddressbyfqdn() an error occured: %s\n",gai_strerror(ret)) ;
    return NULL ;
  }else{
    switch(ai_list->ai_family){
      case (AF_INET):
        t_addr = new IP4Address(&((struct sockaddr_in*)ai_list->ai_addr)->sin_addr);
        break;
      case (AF_INET6):
        t_addr = new IP6Address();
        memcpy (&((IP6Address*)t_addr)->m_addr, &((struct sockaddr_in6*)ai_list->ai_addr)->sin6_addr,sizeof(struct in6_addr));
        break;
      case (AF_UNIX):
        std::cerr << "not implemented yet!" << std::endl ;
        exit(1);
        break;
    }
    std::cerr << t_addr->toString() << std::endl ;
    ai_list = ai_list->ai_next ;
    freeaddrinfo(ai_list);
  }
  return t_addr ;
}


std::string& Resolver::getFQDNbyAddress(Address &addr){
  struct sockaddr *sa;
  struct sockaddr_in sa4 ;
  struct sockaddr_in6 sa6 ;
  void *in46_addr ; //in6_addr and in_addr both
  size_t addr_len ,salen ;
  int ret;
  char s_fqdn[255];

  addr.getRawAddress(&in46_addr,&addr_len);
  switch (addr.getAddressFamily()){
    case(AF_INET):
      salen = sizeof(struct sockaddr_in);
      memset(&sa4,0,salen);
      memcpy( &sa4.sin_addr,in46_addr,addr_len);
      sa4.sin_family = AF_INET ;
      sa = (struct sockaddr*)&sa4 ;
      break;
    
    case(AF_INET6):
      salen = sizeof(struct sockaddr_in6);
      memset(&sa6,0,salen);
      memcpy( &sa6.sin6_addr,in46_addr,addr_len);
      sa6.sin6_family = AF_INET6 ;
      sa = (struct sockaddr*)&sa6 ;
      break;
    
    case(AF_UNIX):
      std::cerr << "not implemented yet!" << std::endl ;
      exit (1);
      break;
  }
  ret = getnameinfo(sa,salen,s_fqdn,255,NULL,0,NI_NOFQDN);
  if (ret){
    std::cerr << gai_strerror(ret);
  }else{
  }
  std::string *ret_fqdn = new std::string(s_fqdn);
  return *ret_fqdn ;

}

/** FIXME  address family can be detected by address */
SocketAddress* Resolver::getSocketAddressByService(const std::string& fqdn,const std::string& port, int type,int af,bool resolve){
  struct addrinfo ai,*ai_list;
  SocketAddress *t_saddr;
  int ret ;
  memset(&ai,0,sizeof(ai));
  ai.ai_flags = AI_ADDRCONFIG ;
  ai.ai_flags |= (resolve ? 0 : AI_NUMERICHOST ) ;
  ai.ai_socktype = type ;
  ai.ai_family = af ;
  std::cerr << "addrinfo_port: " << port << " host: " << fqdn << std::endl ;
  ret = getaddrinfo(fqdn == "any" ? NULL :fqdn.c_str(),port.c_str(),&ai,&ai_list);
  if (ret){
    ;/* throw */
      std::cerr << "addrinfo_error: " << gai_strerror(ret) << std::endl ;
      return NULL ;
  }else{
    t_saddr = new SocketAddress(ai_list->ai_addr);
    freeaddrinfo(ai_list);
  }
  return t_saddr ;

}

SocketAddress* Resolver::getSocketAddressByService(const std::string& fqdn,const int port, int type,int af,bool resolve){

  char buffer[6];
 
  ::sprintf(buffer, "%d", port);
  return Resolver::getSocketAddressByService(fqdn,buffer,type,af,resolve);


}

SocketAddress * Resolver::getSocketAddressByService(const Address & addr, const int port, int type){
  int af = addr.getAddressFamily();
  std::string str_addr = addr.toString();
  return Resolver::getSocketAddressByService(str_addr, port, type, af, false);
}

}
}


