#include "socketaddress.h"
#include "ip4address.h"
#include "ip6address.h"
namespace n2nc {
namespace net {

SocketAddress::SocketAddress()
{
  memset(&this->m_sa,0,sizeof(struct sockaddr_storage));
}

SocketAddress::SocketAddress(struct sockaddr* sa){
  memset(&this->m_sa,0,sizeof(struct sockaddr_storage));
  switch(sa->sa_family){
    case (AF_INET):
      memcpy(&this->m_sa,sa,sizeof(struct sockaddr_in));
      break;
  
    case (AF_INET6):
      memcpy(&this->m_sa,sa,sizeof(struct sockaddr_in6));
      break;
  
    case (AF_UNIX):
      memcpy(&this->m_sa,sa,sizeof(struct sockaddr_un));
      break;
  }
}

SocketAddress::SocketAddress(Address * addr, uint16_t port){
  std::cerr << "not implemented yet" << std::endl ;
//   sockaddr *sa ;
//   size_t salen ;
//   memset(&this->m_sa,0,sizeof(struct sockaddr_storage));
//   addr->getRawAddress((void**)&sa,&salen);
//   memset(&this->m_sa.sa_data,sa,salen);
}

Address* SocketAddress::getAddress(){
  Address *addr;
  struct in_addr *inaddr4 ;
  struct in6_addr *inaddr6 ;
  
  switch(this->m_sa.ss_family){
    case (AF_INET):
      inaddr4 = &((struct sockaddr_in*)&this->m_sa)->sin_addr ;
      addr = new IP4Address( inaddr4 );
      break;
    
    case (AF_INET6):
      inaddr6 = &((struct sockaddr_in6*)&this->m_sa)->sin6_addr ;
      addr = new IP6Address( inaddr6 );
      break;
  
/*    case (AF_UNIX):
      addr = new UnixAddress( &((struct sockaddr_un)this->m_sa).sin_addr);
      break;*/
  }
  return addr ;
}

int SocketAddress::getAddressFamily(){
  return this->m_sa.ss_family ;
}

struct sockaddr* SocketAddress::getRawSockAddr(){
  return (struct sockaddr*)&this->m_sa;
}

int SocketAddress::getRawSockAddrLen(){
  size_t len ;
  switch (this->m_sa.ss_family){
    case(AF_INET):
      len = sizeof(struct sockaddr_in);
      break ;
    case(AF_INET6):
      len = sizeof(struct sockaddr_in6);
      break ;
    case(AF_UNIX):
      len = sizeof(struct sockaddr_un);
      break ;
  }
  return len ;
}

std::string SocketAddress::toString(){
  std::ostringstream ss/*(std::ostringstream::out) FIXME */;
  Address *tmp_addr = this->getAddress();
  uint16_t sinport ;
  struct sockaddr_in* saddr ;
  
  ss << tmp_addr->toString() ;
  /* FIXME this line seems to have problems with gcc -O2 -g0 .. FIXED by cast */
  saddr = (struct sockaddr_in*)&this->m_sa ;
  sinport = ntohs( saddr->sin_port) ;
  ss << " Port: " << sinport ;
//   ss << (uint16_t)::ntohs(  ((struct sockaddr_in*)&this->m_sa)->sin_port) ;
  switch (this->m_sa.ss_family){
    case(AF_INET):
      ss << " AF: IPV4" ;
      break ;
    case(AF_INET6):
      ss << " AF: IPV6" ;
      break ;
    case(AF_UNIX):
      ss << " AF: UNIX" ;
      break ;
  }
  
  delete tmp_addr ;
  
/*  ss << "address: " << ::inet_ntoa( ((struct sockaddr_in*)&this->m_sa)->sin_addr) ;
  ss << " port: " << ::ntohs( ((struct sockaddr_in*)&this->m_sa)->sin_port) ;*/
  return ss.str();
}


SocketAddress::~SocketAddress()
{
}



}
}

