#include "rsa.h"
 
 int pass_cb(char *buf, int size, int rwflag, void *u){
        int len;
        char *tmp;
        /* We'd probably do something else if 'rwflag' is 1 */
        printf("Enter pass phrase for \"%s\"\n", u);

        /* get pass phrase, length 'len' into 'tmp' */
        tmp = "hello";
        len = strlen(tmp);

        if (len <= 0) return 0;
        /* if too long, truncate */
        if (len > size) len = size;
        memcpy(buf, tmp, len);
        return len;
}

namespace n2nc {
namespace security {

Rsa::Rsa() {
  this->m_pub_isloaded = false ;
  this->m_pri_isloaded = false ;
  ::ERR_load_crypto_strings();
}

int Rsa::loadPubFromFile(std::string filename){
  FILE *fp ;
  if ( (fp = fopen(filename.c_str(),"r")) == NULL) { std::cerr << "error loading file" << std::endl ; exit (1) ;}
  
  this->m_pub_rsa = PEM_read_RSAPublicKey(fp,NULL,NULL,NULL);
  if (!this->m_pub_rsa){
    std::cerr << ERR_error_string(ERR_get_error(),NULL) << std::endl ;
    std::cerr << "error loading public key IN PKCS #1 RSAPublicKey" << std::endl ;
    std::cerr << "trying X.509 subjectPublicKeyInfo format(openssl rsa command)" << std::endl ;
    fclose(fp);
    fp = fopen(filename.c_str(),"r");
    this->m_pub_rsa = PEM_read_RSA_PUBKEY(fp,NULL,NULL,NULL);
    if (!this->m_pub_rsa){
      std::cerr << "X.509 subjectPublicKeyInfo format FATAL ERROR ON LOADING." << std::endl ;
      exit (-1) ;
    }

  }
  fclose(fp);
  std::cerr << "RSA Public Key Loaded!!" << std::endl ;
  return this->m_pub_isloaded = true ;
}

int Rsa::loadPriFromFile(std::string filename){
  FILE *fp ;
  if ( (fp = fopen(filename.c_str(),"r")) == NULL) { std::cerr << "error loading file" << std::endl ; exit (1) ;}
  this->m_pri_rsa = PEM_read_RSAPrivateKey(fp,NULL,pass_cb,NULL);
  if (!this->m_pri_rsa){
    std::cerr << ERR_error_string(ERR_get_error(),NULL) << std::endl ;
    std::cerr << "error loading private key" << std::endl ;
    exit(1);
  }
  fclose(fp);
  if(RSA_check_key(this->m_pri_rsa) < 1){
    std::cerr << "Error: RSA Key read failure" << std::endl ;
    exit (1);
  }
  return this->m_pri_isloaded = true ;
}

int n2nc::security::Rsa::getID(){
  
}

int Rsa::encrypt(void * inbuf, void * outbuf, size_t inlen){
  int outw ;
  if (!this->m_pub_isloaded)  exit( -1);
  outw = RSA_public_encrypt(inlen, (u_char*)inbuf, (u_char*)outbuf, this->m_pub_rsa , RSA_PKCS1_OAEP_PADDING);
  if (outw == -1) std::cerr << ERR_error_string(ERR_get_error(), NULL) << std::cerr ;
  return outw ;
}

int Rsa::decrypt(void * inbuf, void * outbuf, size_t inlen){
  int outw ;
  if (!this->m_pri_isloaded)  exit( -1);
  outw = RSA_private_decrypt(inlen, (u_char*)inbuf, (u_char*)outbuf, this->m_pri_rsa , RSA_PKCS1_OAEP_PADDING);
  if (outw == -1) std::cerr << ERR_error_string(ERR_get_error(), NULL) << std::cerr ;
  return outw ;
}


int Rsa::genkey(std::string filename)
{
  std::string name ;
  FILE *fp ;
  
  ::srand(time(NULL));
  
  this->m_pub_rsa = NULL ;
  this->m_pub_rsa = RSA_generate_key(1024,65537UL,NULL,NULL);
  
  name = filename +  "pub" ;
  fp=fopen(name.c_str(),"w");
  PEM_write_RSAPublicKey(fp,this->m_pub_rsa);
  fclose(fp);
  
  name = filename +  "pri" ;
  fp=fopen(name.c_str(),"w");
  PEM_write_RSAPrivateKey(fp,this->m_pub_rsa,NULL,NULL,0,0,NULL);
  fclose(fp);
//   exit (1);
  
}




Rsa::~Rsa() {

}


}
}






