#ifndef N2NC_NETSOCKET_H
#define N2NC_NETSOCKET_H

#include "nixsys.h"
#include "socketaddress.h"
#include "logger.h"

namespace n2nc {
namespace net {

/**
  \brief The socket abstaction class.

  @author fabsoft <fabsoft@gmail.com>
*/
class Socket{
  public:
    ~Socket();
    /** Opens a connection to a remote peer service defined in addr.
        Once using open() function this Socket will be treated as connection-oriented
        socket and by the way this is mandatory on TcpSocket (SOCK_STREAM type).
        NOTE: most functions returns native values.
    */
    virtual int open(SocketAddress &addr);
    /** Send data buf to remote peer */
    virtual int send(void *buf, size_t len);
    /** Check wheter the socket has Received data.Return the lenght of data in the kernel buffer */
    virtual size_t recvCheck();
    /** Receive len leght data from remote peer and store it to buf */
    virtual int recv(void *buf, size_t len);
    /** Closes the socket */
    virtual int close();
    /** Bind the socket to addr */
    virtual int bind(SocketAddress &addr);
    /** Checks if the socket is connected */
    virtual bool isConnected();
    
    typedef int fd_t;
    
    /** @return the address family in OS defined format */
    int getAddressFamily();
    /** @return the type in OS defined format */
    int getType();
    /** return the OS defined format file description (HANDLE in win32) */
    fd_t getFD() const ;
    /** sets boolean socket option
      for now, boolean options only */
    int setSockOpt(int optname,bool value,int level=SOL_SOCKET);
    /** gets a boolean socket option */
    bool getSockOpt(int optname,int level=SOL_SOCKET);
    /** gets the sockaddr (new) associated to local endpoint */
    SocketAddress* getLocalSocketAddress() const;
    /** gets the sockaddr (new) associated to remote endpoint */
    SocketAddress* getRemoteSocketAddress() const;

  protected:
    /** Register a new socket based on address family and type.
        \note Use the derived class Constructor
     */
    Socket(int af, int type);
    /** save some context switch but led memory */
    int   m_fd ;
    int   m_af;
    int   m_type;
    void  *m_buf_dirty ;
//  private:
    Socket();

  /**
    \example socket_ex.cpp A simple Socket example.
  */

};

}
}

#endif
