#ifndef N2NC_NETSOCKETEVENTSHANDLER_H
#define N2NC_NETSOCKETEVENTSHANDLER_H

#include "nixsys.h"
#include "thread.h"
#include "socket.h"
#include "mutex.h"
#include "condition.h"
// #include "socketevents.h"


namespace n2nc {
namespace net {
    
class SocketEvents;

/**
  \brief Provides asynchronus socket managment.
  this class provides a method to handle all events generated by Socket.
  Basically it starts a new thread which poll on any Socket by using select(),
  it generates a callback event defined in SocketEvents interface.
  This way avoid the plenty use of select() and move programmer to a
  event-based socket management.
  \author fabsoft <fabsoft@gmail.com>
*/
class SocketEventsHandler : n2nc::sync::Thread{
  public:
    /** The class should have one instance only.(singleton pattern) */
    SocketEventsHandler();
    ~SocketEventsHandler();
    
    enum check_for_t{
      READ    = 1,  /**< Checks for readable socket */
      WRITE   = 2,  /**< Checks for writeable socket */
      EXCEPT  = 4,  /**< Checks for exception on socket */
      NOTHING = 8   /**< Dont checks the socket. It enable/disable the state. */
    };
    /** Adds a Socket to wait for events.
        If \a se is NULL or omitted, then this Socket \a sock will not use SocketEvents paradigm.
    */
    int add(Socket &sock,check_for_t checkfor, SocketEvents& se);
    /** Removes sock from handler */
    int del(Socket &sock);
    
    bool enableSocket(const Socket& sock);
    bool disableSocket(const Socket& sock);
    
    /** Main loop to catch up all events related to Socket. */
    int waitForEvents();

    struct socket_extra_t{
      check_for_t               check;
      SocketEvents*             se ;
      Socket*                   sock ;
      n2nc::sync::Condition*    wait ;
      int                       last_errno ;
    };
    
    virtual void* entry_point();

  private:
    
    int _main_loop();
    
    int computeMaxFD();
    int tryAddMaxFD(Socket::fd_t fd);
    int tryAddMaxFD(const std::pair<Socket::fd_t,socket_extra_t*>& mypair);
    
    std::map<Socket::fd_t,socket_extra_t*> m_map_new ;
    int m_max_fd;
    
    std::vector<Socket*> m_dispatched ;
    n2nc::sync::Mutex m_lock ;
    int m_wakeup[2];
    pthread_t m_this_thread ;


};




}
}

#endif

